package Demo9;

import java.awt.BorderLayout;


import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.SourceDataLine;


import javax.swing.JFrame;


public class Pdemo extends JFrame {
	public static final int FREQUENCE_ECHANTILLONAGE = 44100;
	public static final int NB_CANALS = 1;
	public static final int OCTETS_PAR_ECHANTILLON = 2;
	public static final int TAILLE_CADRE = NB_CANALS * OCTETS_PAR_ECHANTILLON;
	public static final int NB_OCTETS = FREQUENCE_ECHANTILLONAGE * TAILLE_CADRE;
	public static final int AMPLITUDE_MAXIMUM = (int) (Math.pow(2, OCTETS_PAR_ECHANTILLON * 8 - 1) - 1);

	public static final int BORDURE_X = 450;
	public static final int BORDURE_Y = 350;

	protected Graphique dessin;

	private static final long serialVersionUID = 1L;

	/**
	 * Detail decrivant l'onde.
	 */
	public enum TypeOnde {
		SINUSOIDALE( "sinusoidale" ), 
		TRIANGLE( "triangle" ), 
		TRIANGLE_GENERIQUE( "triangle generique" ), 
		SCIED( "dent scie descendante" ), 
		SCIEM( "dent scie montante" ), 
		CARRE( "carre" ), 
		PULSE_GENERIQUE( "pulse generique" ), 
		BRUIT( "bruit" );
		
		public static TypeOnde [] enumeration = { SINUSOIDALE, TRIANGLE, 
				TRIANGLE_GENERIQUE, SCIED, SCIEM, CARRE, 
				PULSE_GENERIQUE, BRUIT };
		
		private String nom;
		
		private TypeOnde( String nom ) {
			this.nom = nom;
		}
		
		public String toString() {
			return nom;
		}
	}
	
	// Generique au son :
	// valeur entre 20 et 20000
	double frequence = 440.0;
	// valeur entre 0 et 2
	double duree = 1.0;
	
	// Ondes
	TypeOnde typeOnde [] = { TypeOnde.SINUSOIDALE, TypeOnde.SINUSOIDALE }; 
	// valeur entre 0 et 1, utilise si nous avons onde PULSE_GENERIQUE ou TRIANGLE_GENERIQUE.
	double ondeRatio [] = { 0.5, 0.5 };
	
	// Peut etre Onde 2 :
	boolean utilise2Ondes = true;
	// valeur entre 0 et 1
	double ratioVolume = 0.5;
	
	// Filtre :
	// filtreS est entre 0 et 1.
	// filtreA + filtreD <= duree
	boolean utiliseFiltre = true;
	double filtreA = 0.1;
	double filtreD = 0.3;
	double filtreS = 0.5;
	double filtreR = 0.1;
	
	public Onde construireOndeBase( int i ) {
		Onde resultat = null;
		switch( typeOnde[i] ) {
		case SINUSOIDALE :
			resultat = new Sinusoidale( FREQUENCE_ECHANTILLONAGE, frequence );
			break;
		case TRIANGLE :
			resultat = new Triangle( FREQUENCE_ECHANTILLONAGE, frequence );
			break;
		case SCIED :
			resultat = new ScieD( FREQUENCE_ECHANTILLONAGE, frequence );
			break;
		case SCIEM :
			resultat = new ScieM( FREQUENCE_ECHANTILLONAGE, frequence );
			break;
		case CARRE :
			resultat = new Carre( FREQUENCE_ECHANTILLONAGE, frequence );
			break;
		case BRUIT :
			resultat = new Bruit();
			break;
		case PULSE_GENERIQUE:
			resultat = new PulseGenerique( FREQUENCE_ECHANTILLONAGE, frequence, ondeRatio[i] );
			break;
		case TRIANGLE_GENERIQUE:
			resultat = new TriangleGenerique( FREQUENCE_ECHANTILLONAGE, frequence, ondeRatio[i] );
			break;
		}
		resultat.setDure( duree );
		return resultat;
	}
	
	public Onde construireOnde() {
		Onde resultat = construireOndeBase( 0 );
		
		if( utilise2Ondes ){
			resultat = new Mixe( resultat, 1.0 - ratioVolume, construireOndeBase( 1 ), ratioVolume ); 
			resultat.setDure( duree );
		}

		if( utiliseFiltre ){
			ADSR f = new ADSR( FREQUENCE_ECHANTILLONAGE, filtreA, filtreD, filtreS, filtreR );
			f.setDure( duree );
			resultat = new Filtre( resultat, f );
			resultat.setDure( duree );
		}
		
		return resultat;
	}
	
	
	public void jouerNote(Onde onde) {
        AudioFormat format = new AudioFormat(AudioFormat.Encoding.PCM_SIGNED,
                FREQUENCE_ECHANTILLONAGE, OCTETS_PAR_ECHANTILLON * 8, NB_CANALS,
                OCTETS_PAR_ECHANTILLON * NB_CANALS, FREQUENCE_ECHANTILLONAGE, false);

        SourceDataLine line;
        DataLine.Info info = new DataLine.Info(SourceDataLine.class,
                format);
        if (!AudioSystem.isLineSupported(info)) {
            System.err.print("Ligne non supporte par l'ordinateur.");
            System.exit(-1);
        }
        try {
            line = (SourceDataLine) AudioSystem.getLine(info);
            line.open(format);

            byte[] buffer = new byte[NB_OCTETS];

            line.start();

            int j = 0;

            for (Double a : onde) {
                short amp = (short) Math.floor(AMPLITUDE_MAXIMUM * a);
                buffer[j + 0] = (byte) (amp & 0xFF);
                buffer[j + 1] = (byte) ((amp >>> 8) & 0xFF);
                j += TAILLE_CADRE;
                if (j >= NB_OCTETS) {
                    line.write(buffer, 0, NB_OCTETS);
                    j = 0;
                }
            }

            if (j != 0) {
                line.write(buffer, 0, j);
            }

            line.drain();
            line.stop();
            line.close();
        } catch (LineUnavailableException ex) {
            System.err.print("Ligne non disponible.");
            System.exit(-1);
        }
        line = null;
    }

	public void actionsJouerNote() {
		Onde onde = construireOnde();
		dessin.setFonction( onde );
		dessin.repaint();
		jouerNote( onde );
	}
	
	public Pdemo() {
		super( "Synthetiseur" );
		
		setDefaultCloseOperation( JFrame.EXIT_ON_CLOSE );
		setLayout( new BorderLayout() );

		setSize( Graphique.TAILLE_X + BORDURE_X, 2 * Graphique.TAILLE_Y + BORDURE_Y );
	
		/**
		 * Construire l'interface.
		 */

		// CENTER
		dessin = new Graphique( construireOnde() );
		add( dessin, BorderLayout.CENTER );

		/**
		 * Afficher la fenetre.
		 */
		setVisible(true);

		actionsJouerNote();
	}
	
	/**
	 * @param args
	 */
	public static void main(String[] args) {
		javax.swing.SwingUtilities.invokeLater( () -> new Pdemo() );
	}
}
